/*
Dozen Days of Dice episode 3: Threes 
Copyright (c) 2007-2008 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.dice.threes
{
	// Adobe Imports
	import flash.display.SimpleButton;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	
	// Blazing Game imports
	import com.blazinggames.dozendays.dice.*;
	
	/**
	 * The Threes game manager. It handles the game and the players.
	 *
	 * @author Billy D. Spelchan
	 * @version 1.0
	 * Copyright (c) 2007-2008 Blazing Games Inc. All Rights Reserved.
	*/
	public class Threes
	extends MovieClip
	{
// *********************
// ***** CONSTANTS *****
// *********************

		public const NUM_PLAYERS = 6;
		public const STATE_INITIALIZING:int = 0;
		public const STATE_STARTING:int = 1;
		public const STATE_PLAYING:int = 2;
		public const STATE_FINISHING:int = 3;

// *********************
// ***** VARIABLES *****
// *********************

		protected var _playerArray:Array;
		protected var _orderArray:Array;
		protected var _gameState:int;
		protected var _currentOrder:int;
		protected var _currentPlayer:int;

		// note that all _*_movie, _*_btn and _*_txt variables are in movie

// ***********************
// ***** CONSTRUCTOR *****
// ***********************

		public function Threes()
		{
			_playerArray = new Array(
				_redPlayer_movie, _yellowPlayer_movie, _greenPlayer_movie,
				_cyanPlayer_movie, _bluePlayer_movie, _purplePlayer_movie
			);
			_orderArray = new Array(NUM_PLAYERS);
			for (var cntr:int = 0; cntr < NUM_PLAYERS; ++cntr)
				_orderArray[cntr] = cntr;
			_gameState = STATE_INITIALIZING;
			_newRound_btn.addEventListener(
						flash.events.MouseEvent.MOUSE_UP, handleNewRound);
			_title_btn.addEventListener(
						flash.events.MouseEvent.MOUSE_UP, handleQuit);
			startRound();
			stop();
		}
		
// **************************************
// ***** Functions - Event handlers *****
// **************************************

	public function handleNewRound(evt:MouseEvent)
	{
		startRound();
	}
	
	public function handleQuit(evt:MouseEvent)
	{
			var evnt:Event = new Event("RETURNTOTITLE");
			this.dispatchEvent(evnt);
	}

// ***************************************
// ***** Functions - General control *****
// ***************************************

		public function getGame():ThreesGame
		{
			return _game_movie;
		}

		public function startRound()
		{
			var cntr:int;
			var swapwith:int;
			var temp:int;
			
			for (cntr = 0; cntr < NUM_PLAYERS; ++cntr) {
				swapwith = Math.random() * 6;
				temp = _orderArray[cntr];
				_orderArray[cntr] = _orderArray[swapwith];
				_orderArray[swapwith] = temp;
			}
			for (cntr = 0; cntr < NUM_PLAYERS; ++cntr) {
				_playerArray[cntr].setPlayOrder(_orderArray[cntr]+1);
			}
			_currentOrder = -1;
			_newRound_btn.visible = false;
			_title_btn.visible = false;
			_game_movie.visible = true;
			startNextPlayer(false);
		}

		public function startNextPlayer(shotMoon:Boolean)
		{
			var cntr:int;
			
			if (shotMoon)
				_currentOrder = NUM_PLAYERS; 
			++_currentOrder;
			if (_currentOrder >= NUM_PLAYERS) {
				// HANDLE WIN DETERMINATION
				var temp:int;
				var lowestScore:int = _playerArray[cntr].getScore();
				var isATie:Boolean = false;
				var winType:int = 2;
				for (cntr = 1; cntr < NUM_PLAYERS; ++cntr) {
					temp = _playerArray[cntr].getScore();
					if (temp < lowestScore) {
						isATie = false;
						lowestScore = temp;
					} else if (temp == lowestScore)
						isATie = true;
				}
				if (isATie)
					winType = 1;
				for (cntr = 0; cntr < NUM_PLAYERS; ++cntr) 
					if (_playerArray[cntr].getScore() == lowestScore)
						_playerArray[cntr].setResults(winType);
					else
						_playerArray[cntr].setResults(0);
				_newRound_btn.visible = true;
				_title_btn.visible = true;
				_game_movie.visible = false;
			}
			else {
				for (cntr = 0; cntr < NUM_PLAYERS; ++cntr) 
					if (_orderArray[cntr] == _currentOrder) {
						_playerArray[cntr].startPlay(this);
						_currentPlayer = cntr;
					}
			}
		}

	} // end class
} // end package

