/*
Dozen Days of Dice episode 3: Threes 
Copyright (c) 2007-2008 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.dice.threes
{
	// Adobe Imports
	import flash.display.SimpleButton;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	
	// Blazing Game imports
	import com.blazinggames.dozendays.dice.*;
	
	/**
	 * The Threes game. All players share a single instance of this game, but
	 * track their own scoring separately.
	 *
	 * @author Billy D. Spelchan
	 * @version 1.0
	 * Copyright (c) 2007-2008 Blazing Games Inc. All Rights Reserved.
	*/
	public class ThreesGame
	extends MovieClip
	{
// *********************
// ***** CONSTANTS *****
// *********************

		// layout constants
		protected const DIE_SIZE:int = 128;
		protected const NORMAL_Y:int = 0;
		protected const SELECTED_Y:int = 32;

		// player state constants
		public const STATE_STARTING:int = 0;
		public const STATE_WAITING:int = 1;
		public const STATE_KEEPING:int = 2;
		public const STATE_DONE:int = 3;

		// keep state constants
		protected const KEEP_NO:int = 0;
		protected const KEEP_YES:int = 1;
		protected const KEEP_LOCKED:int = 2;

// *********************
// ***** VARIABLES *****
// *********************

		protected var _dice:Array;
		protected var _diceButtons:Array;
		protected var _diceHolding:Array;
		protected var _playerState:int;
		protected var _threesPlayer:ThreesPlayer;
		
		// note that all _*_movie, _*_btn and _*_txt variables are in movie

// ***********************
// ***** CONSTRUCTOR *****
// ***********************

		public function ThreesGame()
		{
			_dice = new Array(_die1_movie, _die2_movie, _die3_movie, 
					_die4_movie, _die5_movie);
			_diceButtons = new Array(_hold1_btn, _hold2_btn, _hold3_btn,
					_hold4_btn, _hold5_btn);
			_diceHolding = new Array(5);
			for (var cntr:int = 0; cntr < 5; ++cntr) {
				_diceButtons[cntr].addEventListener(
						flash.events.MouseEvent.MOUSE_UP, handleHold);
			}
			_roll_btn.addEventListener(
					flash.events.MouseEvent.MOUSE_UP, handleRoll);
			_finish_btn.addEventListener(
					flash.events.MouseEvent.MOUSE_UP, handleRoll);
			changeState(STATE_STARTING);
			stop();
		}
		
// **************************************
// ***** Functions - Event handlers *****
// **************************************

		public function handleRoll(evt:MouseEvent)
		{
			rollTheDice();
		}

		private function rollTheDice()
		{
			var diceRolled:int = 0;
			for (var cntr:int = 0; cntr < 5; ++cntr)
			if (_diceHolding[cntr] != KEEP_NO) 
				_diceHolding[cntr] = KEEP_LOCKED;
			else {
				_dice[cntr].quickRoll();
				++diceRolled;
			}
			if ((diceRolled == 0) && (_threesPlayer != null))
				_threesPlayer.setScore(getScore());
			else
				changeState(STATE_KEEPING);
		}

		public function handleHold(evt:MouseEvent)
		{
			var die:int = (evt.stageX - x)/ DIE_SIZE;
			trace ("Toggling hold for die " + die);
			if (_diceHolding[die] == KEEP_NO) {
				_dice[die].y = SELECTED_Y;
				_diceHolding[die] = KEEP_YES;
			} else if (_diceHolding[die] == KEEP_YES) {
				_dice[die].y = NORMAL_Y;
				_diceHolding[die] = KEEP_NO;
			} else
				trace("error - invalid click. ignored!");
			
			// see if can roll again
			var canRoll:Boolean = false;
			var unkeptDice:int = 0
			for (var cntr:int = 0; cntr < 5; ++cntr) {
				canRoll =  canRoll || (_diceHolding[cntr] == KEEP_YES);
				if (_diceHolding[cntr] == KEEP_NO)
					++unkeptDice;
			}
			canRoll = canRoll && (unkeptDice != 0); 
			_finish_btn.visible = (unkeptDice == 0);
			_roll_btn.visible = canRoll;
		}

// ***************************************
// ***** Functions - General control *****
// ***************************************

		public function startGame(player:ThreesPlayer)
		{
			_threesPlayer = player;
			changeState(STATE_STARTING);
		}

		public function playAI(player:ThreesPlayer)
		{
			_threesPlayer = player;
			var freeCount:int;
			var lowest:int;
			var lowIndex:int;
			var cntr:int;
			var temp:int;
			
			if (_playerState != STATE_KEEPING)
				rollTheDice();
			do {
				// pass 1 find lowest
				lowest = 7;
				lowIndex = -1;
				for (cntr = 0; cntr < 5; ++cntr) {
					temp = _dice[cntr].getValue();
					if (temp == 3) 
						temp = 0;
					if ((_diceHolding[cntr] == KEEP_NO)
							&& (temp < lowest)) {
						lowest = temp;
						lowIndex = cntr;
					}
				}
				if (lowIndex >= 0)
					_diceHolding[lowIndex] = KEEP_YES;
				// pass 2 find all 1s and 3's
				freeCount = 0;
				for (cntr = 0; cntr < 5; ++cntr) {
					temp = _dice[cntr].getValue();
					if ((temp == 1) || (temp == 3))
						_diceHolding[cntr] = KEEP_YES;
					if (_diceHolding[cntr] == KEEP_NO)
						++freeCount;
				}
				rollTheDice();
			} while (freeCount > 0);
		}
		
		/**
		 * calculates the current score of all the held dice.
		 * 
		 * @return the score of held dice. Shooting the moon returns a -1.
		 */
		public function getScore():int
		{
			var theScore:int = 0;
			var temp:int;
			
			for (var cntr:int = 0; cntr < 5; ++cntr)
				if (_diceHolding[cntr] > 0) {
					temp = _dice[cntr].getValue();
					if (temp == 3)
						temp = 0;
					theScore += temp;
				}
			
			if (theScore == 30)
				return -1;
			else
				return theScore;
		}

		/**
		 * calculates the current score of all the dice.
		 * 
		 * @return the score of all dice. Shooting the moon returns a -1.
		 */
		public function getRolledScore():int
		{
			var theScore:int = 0;
			var temp:int;
			
			for (var cntr:int = 0; cntr < 5; ++cntr) {
				temp = _dice[cntr].getValue();
				if (temp == 3)
					temp = 0;
				theScore += temp;
			}
			if (theScore == 30)
				return -1;
			else
				return theScore;
		}
		
		public function changeState(newState:int):void
		{
			var cntr:int;

			_playerState = newState;
			trace ("Changing state to " + newState);
			switch (newState) {
			case STATE_STARTING:
				for (cntr = 0; cntr < 5; ++cntr) {
					_dice[cntr].y = NORMAL_Y;
					_diceHolding[cntr] = KEEP_NO;
					_diceButtons[cntr].visible = false;
				}
				_finish_btn.visible = false;
				_roll_btn.visible = true;
				break;
			case STATE_WAITING:
			case STATE_KEEPING:
				for (cntr = 0; cntr < 5; ++cntr) {
					_diceButtons[cntr].visible = 
							_diceHolding[cntr] != KEEP_LOCKED;
				}
				_finish_btn.visible = false;
				_roll_btn.visible = false;
				break;			
			case STATE_DONE:
			default:
				trace("Unknown State encountered: " + newState);
			}
		}
		
		public function getPlayerState():int
		{
			return _playerState;
		}

	} // end class
} // end package

