/*
Dozen Days of Dice episode 3: Threes 
Copyright (c) 2007-2008 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.dice.threes
{
	// Adobe Imports
	import flash.display.SimpleButton;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	
	// Blazing Game imports
	import com.blazinggames.dozendays.dice.*;
	
	/**
	 * The Threes game manager. It handles the game and the players.
	 *
	 * @author Billy D. Spelchan
	 * @version 1.0
	 * Copyright (c) 2007-2008 Blazing Games Inc. All Rights Reserved.
	*/
	public class ThreesPlayer
	extends MovieClip
	{
// *********************
// ***** CONSTANTS *****
// *********************

		public const RESULTS_NONE:int = 0;
		public const RESULTS_LOSE:int = 0;
		public const RESULT_TIE:int = 1;
		public const RESULT_WIN:int = 2;

// *********************
// ***** VARIABLES *****
// *********************

		protected var _gameScore:int;
		protected var _points:int;
		protected var _isHuman:Boolean;
		protected var _playingGame:Boolean;
		protected var _manager:Threes;

		// note that all _*_movie, _*_btn and _*_txt variables are in movie

// ***********************
// ***** CONSTRUCTOR *****
// ***********************

		public function ThreesPlayer()
		{
			_isHuman = true;
			_playingGame = false;
			_human_btn.addEventListener(flash.events.MouseEvent.MOUSE_UP, 
					handleComputerHumanToggle);
			_computer_btn.addEventListener(flash.events.MouseEvent.MOUSE_UP, 
					handleComputerHumanToggle);
			_human_btn.visible = true;
			_computer_btn.visible = false;
			_gameScore = 0;
			_points = 0;
			stop();
		}

// **************************************
// ***** Functions - Event handlers *****
// **************************************

		public function handleComputerHumanToggle(evt:Event)
		{
			if (_isHuman == true) {
				_isHuman = false;
				_human_btn.visible = false;
				_human_btn.removeEventListener(flash.events.MouseEvent.MOUSE_UP, 
						handleComputerHumanToggle);
				_computer_btn.visible = true;
				_computer_btn.addEventListener(flash.events.MouseEvent.MOUSE_UP, 
						handleComputerHumanToggle);
				if (_playingGame == true)
					_manager.getGame().playAI(this);
			} else {
				_isHuman = true;
				_human_btn.visible = true;
				_human_btn.addEventListener(flash.events.MouseEvent.MOUSE_UP, 
						handleComputerHumanToggle);
				_computer_btn.removeEventListener(flash.events.MouseEvent.MOUSE_UP, 
						handleComputerHumanToggle);
				_computer_btn.visible = false;
			}
		}

// ***************************************
// ***** Functions - General control *****
// ***************************************

		public function setPlayOrder(n:int)
		{
			_orderDie_movie.setValue(n);
			_gameScore = 29;
			_score_txt.text = "Score tbd";
			_flash_movie.visible = false;
			setResults(0);
		}

		public function startPlay(manager:Threes)
		{
			var game:ThreesGame = manager.getGame();
			_manager = manager;
			_flash_movie.visible = true;
			game.startGame(this);
			_playingGame = true;
			if (_isHuman == false)
				game.playAI(this);
		}
		
		public function setScore(n:int)
		{
			_score_txt.text = "Score " + n;
			_gameScore = n;
			_flash_movie.visible = false;
			_playingGame = false;
			_manager.startNextPlayer(n == -1);
		}
		
		public function getScore():int
		{
			return _gameScore;
		}
		
		public function setResults(n:int)
		{
			switch (n)
			{
			case RESULTS_LOSE:
				_results_txt.text = " ";
				break;
			case RESULT_TIE:
				_results_txt.text = "TIE";
				break;
			case RESULT_WIN:
				_results_txt.text = "WIN";
				break;
			}
			if (_gameScore == -1)
				_results_txt.text = "SHOT THE MOON";
			
			_points += n;
			_points_txt.text = "Points " + _points;
		}
	} // end class
} // end package

